
import React from 'react'
import { use, api, app } from 'xadmin'
import { C } from 'xadmin-ui'

import Editor from './editor'

const LogicEditor = props => {
  const { value, onChange, tags, outputMode, ...restEditorProps } = props

  const { t:_t1 } = useI18n()

  const getData = (tags) => {
    let title = outputMode === 'field' ? _t1('字段') : _t1('数据点')
    let data = {
      definitions: {
        data: {
          "type": "object",
          "title": title,
          "description": title,
          "properties": tags ? tags.reduce((p, t) => {
              p[t.id] = {
                "type": "string",
                "format": "string",
                "title": t.name,
                "description": t.description || t.name
              }; 
              return p
            }, {}) : {}
        }
      },
    }
    return data
  }

  return (
    <Editor
      schema={getData(tags)}
      defaultValue={value}
      onChange={onChange}
      width={'100%'}
      height={150}
      outputMode={outputMode || 'raw'}
      {...restEditorProps}
    />
  )
}

const ModelDataFieldEditor = props => {
  return (
    <C is="TagWrap" tableDataId={props.tableData} tableId={props.table?.id}>
      {state => <LogicEditor {...state} {...props} />}
    </C>
  )
}

const ModelDataField = props => {
  const { form } = use('form')
  return <ModelDataFieldEditor value={props.input.value} onChange={props.input.onChange} nodeId={props.field?.tableData} model={form.data} resultType={props.field.resultType || 'any'} />
}

export default ModelDataFieldEditor
export {
  ModelDataField
}
