import { Modal } from 'antd'
import { useRecoilValue, useSetRecoilState } from 'recoil'
import Wrap from './Wrap'
import Formula from './Formula'
import VariableSelect from './VariableSelect'
import React from 'react'
import { keyAtom, isFormulaAtom, errorAtom, stringifyAtom, esprimaAtom } from '../store'
import { formulaTest } from '../utils/expected'
import { getEditorValue } from '../utils/parseAndStringify'


const VariableDlg = (props) => {

  const { visible, value, onChange, onCancel, onForceChange, customStringify, querySchema, getContainer } = props

  const editorRef = React.useRef(null)

  const [ innerValue, setInnerValue ] = React.useState(null)

  const key = useRecoilValue(keyAtom)

  const isFormula = useRecoilValue(isFormulaAtom)
  const setError = useSetRecoilState(errorAtom)
  const stringify = useRecoilValue(stringifyAtom)
  

  // 确定
  const onOk = () => {
    const error = formulaTest(innerValue?.origin || [], isFormula)
    if (error) {
      setError(error)
    } else {
      setError(null)
      onForceChange && onForceChange(_.isString(innerValue) ? innerValue : getEditorValue(innerValue?.origin, stringify, customStringify, isFormula))
      onChange(innerValue)
      onCancel(innerValue)
    }
  }

  const onCancelDlg = () => {
    onForceChange && onForceChange(_.isString(value) ? value : getEditorValue(value?.origin, stringify, customStringify, isFormula))
    onCancel(value)
  }

  React.useEffect(() => {
    setInnerValue(value)
  }, [ value ])

  // const onInnerChange = (v) => {
  //   setInnerValue(v)
  //   onChange(v)
  // }

  return (
    <Modal
      title={_t1("公式编辑")}
      visible={visible}
      width={1000}
      style={{
        height: '80%'
      }}
      onCancel={onCancelDlg}
      onOk={onOk}
      bodyStyle={{
        padding: 0
      }}
      destroyOnClose={true}
      maskTransitionName=""
      transitionName=""
      wrapClassName="iot-editor-variable"
      getContainer={getContainer}
    >
      <div key={key} style={{ height: '100%' }}>
        <Wrap value={value} onChange={(v) => { setInnerValue(v) }} onForceChange={onForceChange} customStringify={customStringify}>
          <div className="iot-editor-variable-dlg-formula">
            <Formula ref={editorRef} onForceChange={onForceChange} disable={!visible} />
          </div>
          <VariableSelect editorRef={editorRef} onForceChange={onForceChange} querySchema={querySchema} customStringify={customStringify} />
        </Wrap>
      </div>
    </Modal>
  )
}

export default VariableDlg
