import _ from 'lodash';
import { atom, selector } from 'recoil'
import { formulaTest } from './utils/expected'
import { editorValueEval } from './utils/parseAndStringify'
import extSchema, { convertFunSchema, convertAttrSchema } from './utils/extSchema'
import { randomString, dataToSchema, schemaToData } from './utils'

/* atom */

const keyAtom = atom({
  key: 'variable.key',
  default: randomString(12)
})

// schema结构
const schemaAtom = atom({
  key: 'variable.schema',
  default: null
})

// formula公式
const formulaAtom = atom({
  key: 'variable.formula',
  default: null
})
// data
const dataAtom = atom({
  key: 'variable.data',
  default: null
})

// 变量及函数信息
const varAndFunAtom = atom({
  key: 'variable.varAndFun',
  default: {}
})

// 选择模式
const selectAtom = atom({
  key: 'variable.select',
  default: null
})

// 节点类型
const nodeTypeAtom = atom({
  key: 'variable.nodeType',
  default: null
})
// 限制
const limitAtom = atom({
  key: 'variable.limit',
  default: null
})
// 加载判断
const hasLoadingAtom = atom({
  key: 'variable.hasLoading',
  default: null
})
// 字符串化
const stringifyAtom = atom({
  key: 'variable.stringify',
  default: false
})
// 函数扩展
const funcExtAtom = atom({
  key: 'variable.funcExt',
  default: []
})
// 属性扩展
const attrExtAtom = atom({
  key: 'variable.attrExt',
  default: []
})
// 公式模式
const isFormulaAtom = atom({
  key: 'variable.isFormulaAtom',
  default: false
})
// 自定义字符串模板解析函数
const customStringifyAtom = atom({
  key: 'variable.customStringify',
  default: null
})

const isSelectFormulaAtom = atom({
  key: 'variable.isSelectFormula',
  default: false
})

const errorAtom = atom({
  key: 'variable.error',
  default: false
})

const formulajsAtom = atom({
  key: 'variable.formulajsAtom',
  default: null
})

const esprimaAtom = atom({
  key: 'variable.esprimaAtom',
  default: null
})

const systemVariableAtom = atom({
  key: 'variable.systemVariableAtom',
  default: null
})

const functionSchemasAtom = atom({
  key: 'variable.functionSchemas',
  default: null
})

const addSystemVariable = (schema, systemVariable=[]) => {
  if (!systemVariable) {
    return schema
  }
  const finalSchema = {
    type: 'object',
    properties: {
      ...(schema?.properties || {}),
    }
  }
  finalSchema.properties['#$time'] = {
    title: _r('系统时间'),
    type: 'string',
    format: 'datetime'
  }
  finalSchema.properties['#$systemVariable'] = {
    title: '系统变量',
    type: 'object',
    properties: {}
  }
  for ( let systemItem of systemVariable ) {
    finalSchema.properties['#$systemVariable'].properties[systemItem.id] = {
      title: systemItem.name,
      type: systemItem.type
    }
    if (systemItem.type === 'date') {
      finalSchema.properties['#$systemVariable'].properties[systemItem.id].type = 'string'
      finalSchema.properties['#$systemVariable'].properties[systemItem.id].format = 'datetime'
    }
  }
  return finalSchema
}


/* selector */

// 最终的schema
const finalSchemaSelector = selector({
  key: 'variable.finalSchema',
  get: ({get}) => {
    const schema = get(schemaAtom)
    const data = get(dataAtom)
    const systemVariable = get(systemVariableAtom)
    if (!schema && data) {
      return dataToSchema(data)
    } else if (schema) {
      const _schema = addSystemVariable(schema, systemVariable)
      return _schema
    } else {
      return { type: 'object', properties: {} }
    }
  },
  cachePolicy_UNSTABLE: { eviction: 'lru', maxSize: 1 }
})

const functionSchemaSelector = selector({
  key: 'variable.functionSchema',
  get: ({get}) => {
    const functionSchemas = get(functionSchemasAtom)
    const isSelectFormula = get(isSelectFormulaAtom)
    const isFormula = get(isFormulaAtom)
    const funcExt = get(funcExtAtom)
    const formulajs = get(formulajsAtom)
    if ((isSelectFormula || isFormula) && formulajs) {
      return functionSchemas || convertFunSchema(funcExt, formulajs)
    } else {
      return []
    }
  },
  cachePolicy_UNSTABLE: { eviction: 'lru', maxSize: 1 }
})

const attrSchemaSelector = selector({
  key: 'variable.attrSchema',
  get: ({get}) => {
    const attrExt = get(attrExtAtom)
    return convertAttrSchema(attrExt)
  },
  cachePolicy_UNSTABLE: { eviction: 'lru', maxSize: 1 }
})

// 预期值
const expectedSelector = selector({
  key: 'variable.expected',
  get: ({get}) => {
    const schema = get(schemaAtom)
    const formula = get(formulaAtom)
    const data = get(dataAtom)
    const evalData = data || schemaToData(schema)
    const result = editorValueEval(formula?.origin, evalData)
    const type = _.isArray(result) ? 'array' : typeof result
    const isFormula = get(isFormulaAtom)
    const error = formulaTest(formula?.origin || [], isFormula)
    
    // const evalInfo = expressionEval(expression, schema, data)
    return { error, result, type, reality: !!data }
  },
  cachePolicy_UNSTABLE: { eviction: 'lru', maxSize: 1 }
})

export { keyAtom, schemaAtom, formulaAtom, dataAtom, varAndFunAtom, selectAtom, nodeTypeAtom, limitAtom, hasLoadingAtom, stringifyAtom, funcExtAtom, attrExtAtom, isFormulaAtom, customStringifyAtom, errorAtom, finalSchemaSelector, expectedSelector, isSelectFormulaAtom, functionSchemaSelector, attrSchemaSelector, formulajsAtom, esprimaAtom, systemVariableAtom, functionSchemasAtom }