import { api, app } from 'xadmin'
//////

/**
 * [画面]查询画面变量
 *
 * ### 使用示例
 *
 * ```
 * const DVariable = getDVariable()
 * ```
 * @returns {Object} 画面变量数据
 */
const getDVariable = async function () {
  const { customVar } = arguments[0]
  return customVar
}

const getDatasetData = async function (id) {
  const { getDataset } = arguments[1]
  return getDataset(id)
}
/**
 * [画面]添加/修改画面变量
 *
 * ### 使用示例
 *
 * ```
 * addEditDVariable('aaa', { bbb: 'ccc' })
 * ```
 * @param {String} key - 画面变量key
 * @param {Any} value - 画面变量值
 */
function addEditDVariable (key, value) {
  const { setCustomVar } = arguments[2]
  setCustomVar(before => ({ ...before, [key]: value }))
}

/**
 * [画面]删除画面变量
 *
 * ### 使用示例
 *
 * ```
 * delDVariable('aaa')
 * ```
 * @param {String} key - 画面变量key
 */
function delDVariable (key) {
  const { customVar, setCustomVar } = arguments[1]
  let result = customVar
  delete result[key]
  setCustomVar(result)
}

//////
/**
 * [画面]修改画面组件属性
 *
 * ### 使用示例
 *
 * ```
 * dashboardUpdateCell('Text_A6CC19AE', { content: 'bgujnbgj' })
 * ```
 * @param {String} key - 画面组件key
 * @param {Object} params - 属性
 */
function dashboardUpdateCell (key, params) {
  const { setCellParams } = arguments[2]
  setCellParams(key, 'base', params)
}

/**
 * [画面]查询系统变量
 *
 * ### 使用示例
 *
 * ```
 * getSVariable().then(data => {
 *    console.log(data)
 * })
 * ```
 * @returns {Array} 系统变量列表
 */
const getSVariable = async () => {
  let result = await api({ name: 'core/systemVariable' })
    .query({ fields: ['value', 'type'] })
    .then(data => data)
    .catch(err => err)
  return result
}

/**
 * [画面]修改系统变量
 *
 * ### 使用示例
 *
 * ```
 * editSVariable({ name: 'aa', uid: 'bb', type: 'number', value: 4 }).then(data => {
 *    console.log(data)
 * })
 * ```
 * @param {Object} item - 修改的系统变量
 * @param {String} item.name - 变量名（必填）
 * @param {Any} item.value - 变量值（类型不可修改）
 * @returns {Object} { status: "OK" } 修改成功
 */
async function editSVariable (item) {
  let result = {}
  const { setSystemVariable } = arguments[1]
  if (!item.name) {
    result = { err: 'name字段为必传项' }
  } else {
    let sList = await api({ name: 'core/systemVariable' })
    .query({ fields: [] }, { where: { name: item.name } })
    .then(data => data)
    .catch(err => err)
    let temp = sList?.items?.[0]
    if (temp) {
      result = await api({ name: 'core/systemVariable' })
        .fetch(`/${temp.id}`, { method: 'PATCH', body: JSON.stringify({ ...temp, ...item }) })
        .then(data => {
          if (data?.json?.status === 'OK') {
            setSystemVariable(before => ({ ...before, [item.name]: item.value }))
            return data.json
          } else {
            return data?.json
          }
        })
        .catch(err => err)
    } else {
      result = { err: '未找到对应系统变量' }
    }
  }
  return result
}

//////

export default {
  getDVariable,
  addEditDVariable,
  delDVariable,
  dashboardUpdateCell,
  getSVariable,
  editSVariable,
  getDatasetData
}
