import React from 'react'
import ReactDOM from 'react-dom'
import Editor from './components/editor'

import mockSchema from '../mock/getTableSchema'

class LocalTestEditor extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            mode: 'text',
            schema: mockSchema,
            logic: '',
            error: ''
        };
    }

    handleChange({value, raw, type}) {
        this.setState({
            logic: JSON.stringify(value),
            error: '',
            raw,
            type
        });
    }

    handleError({error}) {
        this.setState({
            error: error.toString(),
            logic: '',
            raw: ''
        });
    }

    render() {
        try {
            value = JSON.parse(this.state.logic)
        } catch (error) {

        }
        return (
            <div style={{padding: 20}}>
                <h3>公式编辑器 - 返回JSONLogic</h3>
                <div className="editor-wrapper">
                    <Editor
                        defaultValue={{"+":[{"avg":[{"var":["ApiResponse.code"]},25,{"var":["Order"]}]},3]}}
                        outputMode={"jsonlogic"}
                        weight={600}
                        height={300}
                        schema={this.state.schema}
                        ignoreParentKeys={['数据点']}
                        onChange={this.handleChange.bind(this)}
                        onError={this.handleError.bind(this)}
                    />
                </div>
                <h3>公式编辑器 - 返回原始文本</h3>
                <div className="editor-wrapper">
                    <Editor
                        defaultValue={'AVG(ApiResponse.code, 25, Order) + 3'}
                        outputMode={"raw"}
                        weight={600}
                        height={300}
                        schema={this.state.schema}
                        ignoreParentKeys={['数据点']}
                        onChange={this.handleChange.bind(this)}
                        onError={this.handleError.bind(this)}
                    />
                </div>

                <div style={{
                    position: 'fixed',
                    bottom: 0,
                    left: 0,
                    right: 0,
                    zIndex: 10000,
                    padding: 10,
                    background: 'grey'
                }}>
                    <h4>Result <span color='red'>数据类型：{this.state.type}</span></h4>
                    <div style={{color: this.state.error ? 'red': 'black'}}>
                        {this.state.logic || this.state.error}
                    </div>
                </div>
            </div>
        );
    }
}

ReactDOM.render(
    <LocalTestEditor />,
    document.getElementById('app')
);
